/* This file can be best viewed with tab spacing = 2
* in vim ":set ts=2"
*************************************************************************************
*                        Program to draw a geometric figure.
*
* This program creates a frame containing a toolbar and a canvas.
* A rectangle can be drawn on the canvas between two points.
* The points on the canvas can be selected by mouse clicks.
* The toolbar contains a comboBox and an uneditable text field.
* The comboBox is used to set the colour of the rectangle.
* Currently the text field is unused.
*
* Extend/modify the program to add following features:
* 1. Display the current co-ordinate of mouse(on the canvas) in the text field present in toolBar.
* 2. Add one more comboBox to select current geometric shape amongst line, rectangle, and ellipse. 
* 3. Add a push button which randomly chooses the colour of the figure from color comboBox.
*
* Hints: Graphics.drawLine(), Graphics.drawOval(), Random.nextInt()
***************************************************************************************/

/* import required packages */
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Random;

/**
 * Class ToolBar 
 * drawingCanvas : Instance of the DrawingCanvas object
 */
class ToolBar extends JToolBar implements ActionListener, MouseMotionListener {

	static final public Color[] color ={Color.white, Color.black, Color.blue, Color.red, Color.green, Color.yellow};
	static final private String[] COLORNAMES = {"White","Black","Blue","Red","Green","Yellow"};
	static final private String BOXCOLOR = "ccolor";

	static final private String[] SHAPES = {"line", "rectangle", "ellipse"};
	static final private String SHAPEAXN = "cshape";

	DrawingCanvas drawingCanvas;	
	Random random;

	public static JTextField textField;


	/**
	 * Constructor adds the widgets(controls) in the toolbar
	 */
	public ToolBar(){
    addWidgets();
	random = new Random();	
	addMouseMotionListener(this);
	}//end of constructor

	/**
	 * canvas : canvas in use
	 */
	public void setDrawingCanvas(DrawingCanvas canvas){
		drawingCanvas = canvas;
	}//end of setDrawingCanvas

	/**
	 * This function adds widgets in the toolbar. 
	 */
	protected void addWidgets() {
				JComboBox comboBox = null;
				JComboBox comboBox1 = null;
				JButton btn = new JButton("Color");
				btn.addActionListener(this);

		  
				//comboBox for colour
			  comboBox= makeComboBox(COLORNAMES,BOXCOLOR); 
			  comboBox1= makeComboBox(SHAPES,SHAPEAXN); 
		add(btn);
        add(comboBox);
        add(comboBox1);
			
				//textField
				textField = makeTextField(); 
        add(textField);

	}//end of addWidgets()

	/**
	 * Create and initialize the comboBox
	 * text : Items in the comboBox
	 * actionCommand : text to set actionCommand of comboBOx 
	 * @JComboBox : crated comboBox 
	 */
	protected JComboBox makeComboBox(String[]text , String actionCommand){
        JComboBox comboBox = new JComboBox(text);
        comboBox.setActionCommand(actionCommand);
        comboBox.addActionListener(this);
        
				return comboBox;
	}//end of makeComboBox()

	/**
	 * Create and initialize the textField
	 */
	protected JTextField makeTextField(){
        JTextField textField = new JTextField();
				textField.setEditable(false);

				return textField;
	}//end of makeTextField()

	/**
	 * e : action received 
	 */
	public void actionPerformed(ActionEvent e) {
		System.out.println(e.getActionCommand());
		if(e.getActionCommand().equalsIgnoreCase(BOXCOLOR)){
				drawingCanvas.setColor(color[((JComboBox)getComponentAtIndex(1)).getSelectedIndex()]);
				drawingCanvas.repaint();
			}
		if(e.getActionCommand().equalsIgnoreCase(SHAPEAXN)){
			drawingCanvas.setShape(SHAPES[((JComboBox)getComponentAtIndex(2)).getSelectedIndex()]);
			drawingCanvas.repaint();
		}
		else{
				int i = random.nextInt(6);
				drawingCanvas.setColor(color[i]);
				((JComboBox)getComponentAtIndex(1)).setSelectedIndex(i);
				drawingCanvas.repaint();
		}

	}//end of actionPerformed()
		public void mouseDragged(MouseEvent e){
		}
		public void mouseMoved(MouseEvent e){
			((JTextField)getComponentAtIndex(3)).setText(""+e.getX()+","+e.getY());
			updateUI();
		}

}//end of class ToolBar

/**
 * Canvas for drawing figures
 * p1, p2 : Stores two corner points of the rectangle to be drawn
 * color : store color in which rectangle is drawn
 */
class DrawingCanvas extends Canvas implements MouseListener, MouseMotionListener{

	Point p1,p2,temp;
	Color color;	
	String shape = null;

	/**
	 * Sets event handlers and initializes p1, p2 and color
	 */
	public DrawingCanvas(){
		addMouseMotionListener(this);
		addMouseListener(this);
		color= Color.white;
		p1 = new Point();
		p2 = new Point();
		temp = new Point(-1,-1);
	}//end of constructor

	public void setColor(Color c){
		color = c;
	}//end of setColor()

	public void setShape(String c){
		shape = c;
	}//end of setColor()

	public void mouseMoved(MouseEvent me) {
		ToolBar.textField.setText(""+me.getX()+","+me.getY());
		 ToolBar.textField.updateUI();
	}//end of mouseMoved()

	/** 
	 * Stores coordinates of topLeft and bottomRight corners of the rectangle in Points p1 and p2.
	 */
	public void mouseClicked(MouseEvent me) {
		if(temp.x==-1){
			temp.setLocation(me.getX(),me.getY());
		}
	  else{
			p2.setLocation(me.getX(),me.getY());
			p1.setLocation(temp);
			repaint();
		}
	}//end of mouseClicked()
 
	public void mouseDragged(MouseEvent me) {
	}
 
	public void mouseEntered(MouseEvent me) {
	}
  
	public void mouseExited(MouseEvent me) {
	}
  
	public void mousePressed(MouseEvent me) {
	}
  
	public void mouseReleased(MouseEvent me) {
	}

	/** Method paint(): paints the surface of the canvas.
	 *	Draws a rectangle in color currently selected in comboBox in the toolbar.
	 *	Position of the rectangle is obtained from the points p1 and p2.
	 */
	public void paint(Graphics g){
			int x,y;
			x = (p1.x<p2.x)?p1.x:p2.x;
			y = (p1.y<p2.y)?p1.y:p2.y;
			
			temp.setLocation(-1,-1);

			g.setColor(color);
			if(shape != null && shape.equalsIgnoreCase("rectangle")){
				g.drawRect(x,y,Math.abs(p1.x-p2.x),Math.abs(p1.y-p2.y));
			}
			else if(shape != null && shape.equalsIgnoreCase("ellipse")){
				System.out.println(p1.x+" "+p1.y+" "+p2.x+" "+p2.y);
				g.drawOval(p1.x, p1.y, Math.abs(p1.x-p2.x), Math.abs(p1.y-p2.y));
			}
			else {
				g.drawLine(p1.x, p1.y, p2.x, p2.y);
			}
	
	}//end of paint()
 
}//end of class DrawingCanvas

/** Class: Q1
 * Extends JPanel to include a toolbar and a canvas into it
 * An instance of this class is set as content pane of main window of the application.
 */
public class Q1 extends JPanel{ 
	public Q1() {
		super(new BorderLayout());

		//Create the toolbar.
		ToolBar toolBar = new ToolBar();
		//Create the canvas.
		DrawingCanvas drawingCanvas = new DrawingCanvas();
		//Lay out the main panel.

		toolBar.setDrawingCanvas(drawingCanvas);
		toolBar.setFloatable(false);

		// Size the panel
		setPreferredSize(new Dimension(450, 130));
	
		// Add the canvas and the toolbar
		add(drawingCanvas);
		add(toolBar, BorderLayout.PAGE_END);
	}

	 /**
     * Create the GUI and show it.  
		 * Creates a JFrame and adds an instance of Panel Q1 to its content pane.
     */
	private static void createAndShowGUI() {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
	 			JFrame frame = new JFrame("Q1");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the drawing pane.
        Q1 drawingPane = new Q1();
        drawingPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(drawingPane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }//end of createAndShowGUI() 

    public static void main(String[] args) {
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }//end of main()
}//end of class Q1
