/* The file is best viewed with tab spacing =2
* in vi ":set ts=2"
**************************************************************************************
*                 Program to write a record in text file and database
* This program writes a record of type Entry in a text file and database table.
* The class Entry is used to hold the record.
* Class FileIO provides the method insertEntry() to write an entry in the file.
* Class DataBaseIO creates a table of schema Entry.
* It also provides the method insertEntry() to insert a record in the table.
*
* Extend/modify the program for
* 1. Reading the entries from the text file and insert them into database.
*    (The format of input file is same as "sample.txt".)
* 2. Read the entries from the database table sorted by name and 
*	   write the sorted entries in output file "out.txt".
*
* Create a table with following definetion in your database.
* CREATE TABLE dummy ( no INTEGER(9), name CHAR(20) , dept CHAR(5), salary FLOAT(10,2));
*
* Hint: StringTokenizer class available in java for string parsing.
**************************************************************************************/

/* import required packages */
import java.io.*;
import java.util.*;
import java.sql.*;
import java.text.*;

/** 
 * Class Entry: entry is used to hold a record 
 */
class Entry{
	public	int empNumber;
	public String name;
	public String department;
	public float salary;
	
	/**
	 * constructor to initilize the memeber veriables
	 */
	public Entry(){
		empNumber=-1;
		name= new String("");
		department = new String("");
		salary= 0;
	}// end of constructor

}//end of class Entry

	
/** 
 * FileIO: Class to handle file I/O.
 * getEntry(): read current entry from text file and move the cursor to next line
 * insertEntry(): append entry in a text file
 */
class FileIO{
	public static final int READ=1;
	public static final int WRITE=2;

	private BufferedReader fileReader;
	private FileOutputStream fileWriter;
	
	/**
	 * constructor opens the file in append mode if mode=WRITE
	 * else opens the file in read mode and skips the first line of the text
	 * fname: Name of the file
	 * mode : READ or WRITE 
	 */
	public FileIO(String fname, int mode) {
		try	{
			if(mode==WRITE)
				fileWriter = new FileOutputStream(fname,true);//open file in append mode
			else
				fileReader = new BufferedReader(new FileReader(fname));
		}
		catch (Exception e){
			System.out.println("error: can not open file");
		}
	}//end of Constructor
	
	/** Signature of method getEntry()
	 * @Entry: entry read from the file
	 */
public Entry getEntry(){
		if(fileReader == null)return null;
		Entry ent = new Entry();
		String nextLine = null;
		try{
		nextLine = fileReader.readLine();
		}catch(Exception e){
			return null;
			}
		if(nextLine == null) return null;
		StringTokenizer st = new StringTokenizer(nextLine);
		ent.empNumber = Integer.parseInt(st.nextToken());
		ent.name = st.nextToken();
		ent.department= st.nextToken();
		ent.salary = Float.parseFloat(st.nextToken());
		return ent;
	}//end of function getEntry()

	/** Method to write an entry in file
 	 * ent: entry to be written
	 */
	public void insertEntry(Entry ent){
		try	{
			String record = new String("" + ent.empNumber + "\t" + ent.name + "\t"+ent.department+"\t" + ent.salary+"\n");
			fileWriter.write(record.getBytes());
		}
		catch (IOException e){
			System.out.println( "IO error:" + e );
		} 
	}//end of function writeFile
	
	/**
	 * Method to close opened files.
	 */
	protected void finalize(){
		try{
		if(fileWriter !=null) 	
			fileWriter.close();
		else if(fileReader !=null)	
			fileReader.close();
		}catch(Exception e) {
			System.out.println("Error closing file.");
			e.printStackTrace();
		}
	}// end of finalize

}//End of class FileIO


/**
 * Class for database handling I/O.
 * insertEntry(): add entry in the table
 */
class DataBaseIO{
	public static final String empNumber="no"; 
	public static final String name = "name";
	public static final String department = "dept";
	public static final String salary = "salary";

	private Statement stmt;

	private static Connection conn =  null;
	private static Properties dbProps = null;
	private static String confFile = "db.conf";

	private static void loadDriver ()
				throws Exception {
						String driver = "org.gjt.mm.mysql.Driver";
						Class.forName(driver);
				}

	private void loadProperties(String propFileName)
			throws IOException {
				FileInputStream fis = new FileInputStream(propFileName);
				dbProps = new Properties();
				dbProps.load(fis);
		}

	/**
	 * Constructor
	 * opens connection with database 
	 * set cs701 as current databse
	 * create a table of schema Entry
	 */
	public DataBaseIO(){
		try{
			loadDriver();
			loadProperties(confFile);
			String dbHost = dbProps.getProperty("dbhost");
			String dbUser = dbProps.getProperty("dbuser");
			String dbPassword = dbProps.getProperty("dbpassword");
			String dbName = dbProps.getProperty("dbname");
			String dbType = dbProps.getProperty("dbtype");

			String jdbcString = "jdbc:"+dbType+"://"+dbHost+"/"+dbName+"?user="+dbUser+"&password="+dbPassword;

			if(conn == null){
				conn = DriverManager.getConnection(jdbcString);
				stmt = conn.createStatement();
			}
			else if(conn.isClosed()){
				conn=DriverManager.getConnection(jdbcString);
				stmt = conn.createStatement();
			}
		} 
		catch (Exception e){
			e.printStackTrace(System.out);
		}
			
	}//end of constructor


	/** Method to insert an entry into the table.
	 * entry: entry to be inserted
	 */
	public void insertEntry(Entry ent){
		try{
			String str="insert into dummy values ("+
					ent.empNumber+",\'"+ent.name+"\' , \'"+ent.department+"\' , "+ent.salary+")";
			stmt.execute(str);
		}
		catch(Exception ex){
			ex.printStackTrace();
		}
	}//end of function insertEntry

	public Entry[] getDBEntries(){

		String query = "select * from dummy order by name";
		Vector vec = new Vector();
		try{
		ResultSet rs = stmt.executeQuery(query);
		if(rs.first()){
			do{
				Entry Ent = new Entry();
				Ent.empNumber = rs.getInt(empNumber);
				Ent.name = rs.getString(name);
				Ent.department = rs.getString(department);
				Ent.salary = rs.getFloat(salary);
				vec.add(Ent);
			}while(rs.next());
			return (Entry[])vec.toArray(new Entry[0]);
		}
		}catch(Exception E){
			E.printStackTrace();
			return null;
		}
		return null;
	
	}
	

	/**
	 * Method to close connection with the database.
	 */
	protected void finalize(){
		try{
			conn.close();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}// end of finalize

}//end of class DataBaseIO


/**
 * Class with main() function
 * Write an entry into a file and also inserts it into database
 */
public class Q2
{
	public static void main(String args[]) 
	{

		FileIO fw1 = new FileIO("sample.txt",FileIO.READ);
		DataBaseIO db = new DataBaseIO();
		while(true){
			Entry Ent = fw1.getEntry();
			if(Ent==null){
				break;
			}
			db.insertEntry(Ent);
		}

		
		FileIO fw = new FileIO("out.txt",FileIO.WRITE);
		Entry[] ents = db.getDBEntries();
		if(ents != null ){
		for(int i=0; i<ents.length; i++){
			fw.insertEntry(ents[i]);
		}
		}

	}//end of function main()
}//end of class Q2
